﻿-- RSwapper 3: Swaps main-hand weapon for a rogue, depending on stealth, stuns and other buffs
--
-- Based on the rswapper code by Shambarimen (Muradin), who rewrote the code
-- for the original idea of RogueSwapper by Krax (Daggerspine) aka m0rfin.
--
-- This mod depends on certain Ace-3 libraries (included)
-- Other than that, this mod is self-sufficient (no other depends)

local RSwapper3 = LibStub("AceAddon-3.0"):GetAddon("RSwapper3");
local L = LibStub("AceLocale-3.0"):GetLocale("RSwapper3", true);

local RS3GUI = RSwapper3.RS3GUI;
local dataobj = RSwapper3.DataObj;

-- Global Functions Made Local
local GetSpellInfo, GetShapeshiftForm, UnitChannelInfo, GetInventoryItemLink, GetItemInfo, GetTime, PlaySoundFile, UnitBuff, UnitDebuff, ShowUIPanel, HideUIPanel, GetContainerNumSlots, GetContainerItemLink, strfind
	= GetSpellInfo, GetShapeshiftForm, UnitChannelInfo, GetInventoryItemLink, GetItemInfo, GetTime, PlaySoundFile, UnitBuff, UnitDebuff, ShowUIPanel, HideUIPanel, GetContainerNumSlots, GetContainerItemLink, strfind;

local LTARGETSTUNS = RSwapper3.LTARGETSTUNS;
local LPLAYERSTUNS = RSwapper3.LPLAYERSTUNS;
local Sounds = RSwapper3.Sounds;

local PlayerGUID = UnitGUID("player");

-- LASTERR List (Just for reference)
-- -22 Sword Not Set
-- -21 Sword Not Weapon (Not Cached)
-- -12 Dagger Not Set
-- -11 Dagger Not Weapon (Not Cached)
-- 0 No Error
-- 1 Dagger Swap Pending
-- 2 Sword Swap Pending
-- 3 Was Fishing
-- 10 Dagget Not Found
-- 14 Dagger Is Off Hand, but Main Hand is Main Hand Only
-- 15 Dagger Is Broken
-- 20 Sword Not Found
-- 24 Sword Is Off Hand, but Main Hand is Main Hand Only
-- 25 Sword Is Broken

-- Main Functions

function RSwapper3:OnInitialize()
	RSwapper3.defaults = {
		char = {
			Dagger = nil,
			Sword = nil,
			ODagger = nil,
			OSword = nil,
			Stealth = false,
			Dance = false,
			Blind = false,
			Enable = true,
			Offhand = true,
			Sound = true,
		},
	}
	RSwapper3.db = LibStub("AceDB-3.0"):New("RSwapper3DB", RSwapper3.defaults);
end

function RSwapper3:OnEnable()
	local playerclass, _;
	_, playerclass = UnitClass("player");
	RSwapper3.ISCHARROGUE = (playerclass == "ROGUE");
	if (not RSwapper3.ISCHARROGUE) then
		RSwapper3.DOTip = L["You're not a rogue. RSwapper3 will be dormant."];
		RSwapper3:Print(RSwapper3.DOTip);
		return;
	end

	RSwapper3:Print(L["Rogue Detected!"]);
	local options = {
		name = "RSwapper3",
		handler = RSwapper3,
		type = 'group',
		args = {
			[L["Enable"]] = {
				order = 0,
				type = "toggle",
				name = L["Enable"],
				desc = function()
					if (RSwapper3.db.char.Enable) then
						return L["|cff00ff00[Enabled]|r Enable/Disable RSwapper3 for this character."];
					else
						return L["|cffff0000[Disabled]|r Enable/Disable RSwapper3 for this character."];
					end
				end,
				set = "SetEnable",
				get = function(info)
					return RSwapper3.db.char.Enable
				end,
			},

			[SOUND_LABEL] = {
				order = 1,
				type = "execute",
				name = SOUND_LABEL,
				desc = function()
					if (RSwapper3.db.char.Sound) then
						return L["|cff00ff00[On]|r"] .. " " .. ENABLE_SOUNDFX;
					else
						return L["|cffff0000[Off]|r"] .. " " .. ENABLE_SOUNDFX;
					end
				end,
				func = 'SetSound',
			},

			[L["Dagger"]] = {
				order = 2,
				type = "input",
				name = L["Dagger"],
				desc = function()
					if (RSwapper3.db.char.Dagger) then
						return (RSwapper3.db.char.Dagger .. L[" Set the dagger with an item link."]);
					else
						return L["|cffff0000[Not Set]|r Set the dagger with an item link."];
					end
				end,
				set = 'SetDagger',
				get = function(info)
					return RSwapper3.db.char.Dagger
				end,
			},

			[L["Sword"]] = {
				order = 3,
				type = "input",
				name = L["Sword"],
				desc = function()
					if (RSwapper3.db.char.Sword) then
						return (RSwapper3.db.char.Sword .. L[" Set the sword with an item link."]);
					else
						return (L["|cffff0000[Not Set]|r Set the sword with an item link."]);
					end
				end,
				set = 'SetSword',
				get = function(info)
					return RSwapper3.db.char.Sword
				end,
			},

			[L["Offhand"]] = {
				order = 4,
				type = "execute",
				name = L["Offhand"],
				desc = function()
					if (RSwapper3.db.char.Offhand) then
						return L["|cff00ff00[Checking Off-Hand then in Bags]|r"] .. L[" Toggle looking for weapons in Off-Hand slot."];
					else
						return L["|cff996633[Checking only in Bags]|r"] .. L[" Toggle looking for weapons in Off-Hand slot."];
					end
				end,
				func = 'SetOffhand',
			},

			[L["Stealth"]] = {
				order = 5,
				type = "execute",
				name = L["Stealth"],
				desc = function()
					if (RSwapper3.db.char.Stealth) then
						return L["|cff666666[Swapping only in stealth!]|r"] .. L[" Toggle swapping when your target is stunned."];
					else
						return  L["|cffff4646[Swapping in stealth and during stuns!]|r"] .. L[" Toggle swapping when your target is stunned."];
					end
				end,
				func = 'SetStealth',
			},

			[L["Dance"]] = {
				order = 6,
				type = "execute",
				name = L["Dance"],
				desc = function()
					if (RSwapper3.db.char.Dance) then
						return L["|cff00ff00[On]|r"] .. L[" Toggle swapping during Shadow Dance buff."];
					else
						return L["|cffff0000[Off]|r"] .. L[" Toggle swapping during Shadow Dance buff."];
					end
				end,
				func = 'SetDance',
			},

			[L["BlindOption"]] = {
				order = 7,
				type = "execute",
				name = L["BlindOption"],
				desc = function()
					if (RSwapper3.db.char.Blind) then
						return L["|cff00ff00[On]|r"] .. L[" Toggle announcing when you cast Blind via Say."];
					else
						return L["|cffff0000[Off]|r"] .. L[" Toggle announcing when you cast Blind via Say."];
					end
				end,
				func = 'SetBlind',
			},

		},
	}
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RSwapper3", options, {"rswapper3","rswapper"});

	if (not RSwapper3.db.char.Enable) then
		RSwapper3:Print(format(L["RSwapper3 is |cffff0000disabled|r! Use \"/rswapper3 %s\" to enable it."], L["Enable"]));
	end

	RSwapper3:SwapIfNeeded(true);

	RSwapper3:RegisterEvent("UNIT_AURA");
	RSwapper3:RegisterEvent("UNIT_INVENTORY_CHANGED");
	RSwapper3:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
	RSwapper3:RegisterEvent("PLAYER_TARGET_CHANGED");
	if (RSwapper3.db.char.Blind) then
		RSwapper3:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	end

	RS3GUI:DoInit();

	SlashCmdList['RSwapper3_GUI'] = RS3GUI.Toggle;

	_G['SLASH_RSwapper3_GUI1'] = '/rswapperg';
	_G['SLASH_RSwapper3_GUI2'] = '/rs3g';
end

function RSwapper3:OnDisable()
	-- Called when the addon is disabled
end

-- WeaponExists returns (number found, string link, bool broken)
-- possible returns for found
-- -3 - Zoning; sometimes just prior to switching zones, inventory information is not available.
-- -2 - weaponname was nil or empty
-- -1 - not weapon
-- 0 - not found
-- 1 - main-hand
-- 2 - off-hand
-- 3 - bag
-- 4 - off-hand, but the current main-hand is main-hand only (swapping will unequip current main-hand)
-- isoffhand is true if the weapon is supposed to be an Off Hand, then the Off Hand option toggles Main Hand search
-- isoffhand also reverses the returns 1 and 2, 1 means it is in off-hand, 2 means main-hand
-- isoffhand makes 4 unused.
-- skip is the link to the main-hand, so if isoffhand, we don't use that link
function RSwapper3:WeaponExists(weaponname, isoffhand, skip)
	local itemname, itemlink, itemtype, invname, invlink, mainloc, broken, dur, durmax, zoning, _;

	if ((weaponname == nil) or (string.len(weaponname) == 0)) then
		return -2;
	end

	itemname, itemlink, _, _, _, itemtype = GetItemInfo(weaponname);
	if ((itemlink == nil) or (itemtype ~= RSwapper3.LWEAPON)) then
		return -1;
	end

	local firstslot, secondslot;
	if (isoffhand) then
		firstslot = RSwapper3.nSECONDARYHANDSLOT;
		secondslot = RSwapper3.nMAINHANDSLOT;
	else
		firstslot = RSwapper3.nMAINHANDSLOT;
		secondslot = RSwapper3.nSECONDARYHANDSLOT;
	end

	-- check the main-hand
	invlink = GetInventoryItemLink("player", firstslot);
	if (invlink) then
		invname, invlink, _, _, _, _, _, _, mainloc = GetItemInfo(invlink);
		if ((itemname == invname) and (invlink ~= skip)) then
			broken = (GetInventoryItemBroken("player", firstslot) == 1);
			return 1, invlink, broken;
		end
	else
		zoning = true; -- we MIGHT be zoning, slot may not actually be empty
	end
	-- mainloc will be nil now, if isoffhand, and not checking main-hand.
	-- But no worries, since we won't be swapping from main hand
	-- So no restriction error

	-- check the off-hand
	if (RSwapper3.db.char.Offhand) then
		invlink = GetInventoryItemLink("player", secondslot);
		if (invlink) then
			invname, invlink = GetItemInfo(invlink);
			if ((itemname == invname) and (invlink ~= skip)) then
				broken = (GetInventoryItemBroken("player", secondslot) == 1);
				-- The weapon was found in the off-hand, check whether the main-hand needs to stay main-hand
				if ((not isoffhand) and (mainloc == "INVTYPE_WEAPONMAINHAND")) then
					return 4, invlink, broken;
				else
					return 2, invlink, broken;
				end
			end
		end
	end

	-- scan bags
	for bag = 0, NUM_BAG_FRAMES do
		for slot = 1, GetContainerNumSlots(bag) do
			invlink = GetContainerItemLink(bag, slot);
			if (invlink) then
				invname, invlink = GetItemInfo(invlink);
				dur, durmax = GetContainerItemDurability(bag, slot);
				if ((itemname == invname) and (invlink ~= skip)) then
					broken = (dur == 0);
					return 3, invlink, broken;
				end
			end
		end
	end

	if ((zoning) and (not GetBagName(0))) then -- GetBagName(0) should always return "Backpack" localized, but nil if inventory information is unavailable.
		return -3;
	end

	-- didn't find
	return 0;
end

function RSwapper3:SetDagger(info, dag)
	local found, link, broken;

	if ((dag == nil) or (string.len(dag) == 0)) then
		RSwapper3:Print(format(L["Usage: /rswapper3 %s [%s Item Link]"], L["Dagger"], L["Dagger"]));
		return;
	end

	found, link, broken = RSwapper3:WeaponExists(dag);
	if (found == -1) then
		RSwapper3:Print(format(L["%s |cffff0000does not appear to be a weapon!|r"], dag));
		return;
	elseif (found == 0) then
		RSwapper3:Print(format(L["|cffff0000Could not find|r %s"], dag));
		return;
	else
		RSwapper3.db.char.Dagger = link;
		RSwapper3:Print(format(L["Dagger set to: %s"], RSwapper3.db.char.Dagger));
		RS3GUI:UpdateTab(1);
	end

	if (broken) then
		RSwapper3:Print(format(L["|cffff0000Warning:|r %s is broken!"], link));
	end
end

function RSwapper3:SetSword(info, sword)
	local found, link, broken;

	if ((sword == nil) or (string.len(sword) == 0)) then
		RSwapper3:Print(format(L["Usage: /rswapper3 %s [%s Item Link]"], L["Sword"], L["Sword"]));
		return;
	end

	found, link, broken = RSwapper3:WeaponExists(sword);
	if (found == -1) then
		RSwapper3:Print(format(L["%s |cffff0000does not appear to be a weapon!|r"], sword));
		return;
	elseif (found == 0) then
		RSwapper3:Print(format(L["|cffff0000Could not find|r %s"], sword));
		return;
	else
		RSwapper3.db.char.Sword = link;
		RSwapper3:Print(format(L["Sword set to: %s"], RSwapper3.db.char.Sword));
		RS3GUI:UpdateTab(1);
	end

	if (broken) then
		RSwapper3:Print(format(L["|cffff0000Warning:|r %s is broken!"], link));
	end
end

function RSwapper3:SetOffhand()
	if (RSwapper3.db.char.Offhand) then
		RSwapper3:Print(L["|cff996633[Checking only in Bags]|r"]);
	else
		RSwapper3:Print(L["|cff00ff00[Checking Off-Hand then in Bags]|r"]);
	end
	RSwapper3.db.char.Offhand = not RSwapper3.db.char.Offhand
	RS3GUI:UpdateTab(2);
end

function RSwapper3:SetEnable(info, value)
	if (RSwapper3.db.char.Enable) then
		RSwapper3:Print(L["RSwapper3 has been |cffff0000disabled|r for this character."]);
		RSwapper3:PlaySound(Sounds.Disable);
	else
		RSwapper3:Print(L["RSwapper3 has been |cff00ff00enabled|r for this character."]);
		RSwapper3:PlaySound(Sounds.Enable);
	end
	RSwapper3.db.char.Enable = not RSwapper3.db.char.Enable;
	RSwapper3:SwapIfNeeded();
	RS3GUI:UpdateTab();
end

function RSwapper3:SetSound()
	if (RSwapper3.db.char.Sound) then
		RSwapper3:Print(ENABLE_SOUNDFX .. " " .. L["|cffff0000[Off]|r"]);
	else
		RSwapper3:Print(ENABLE_SOUNDFX .. " " .. L["|cff00ff00[On]|r"]);
	end
	RSwapper3.db.char.Sound = not RSwapper3.db.char.Sound;
	RSwapper3:PlaySound(Sounds.Sound);
	RS3GUI:UpdateTab(2);
end

function RSwapper3:SetStealth()
	if (RSwapper3.db.char.Stealth) then
		RSwapper3:Print(L["|cffff4646[Swapping in stealth and during stuns!]|r"]);
	else
		RSwapper3:Print(L["|cff666666[Swapping only in stealth!]|r"]);
	end
	RSwapper3.db.char.Stealth = not RSwapper3.db.char.Stealth;
	RSwapper3:SwapIfNeeded();
	RS3GUI:UpdateTab(2);
end

function RSwapper3:SetDance()
	if (RSwapper3.db.char.Dance) then
		RSwapper3:Print(L["Swapping during Shadow Dance buff is |cffff0000disabled|r."])
	else
		RSwapper3:Print(L["Swapping during Shadow Dance buff is |cff00ff00enabled|r."])
	end
	RSwapper3.db.char.Dance = not RSwapper3.db.char.Dance;
	RSwapper3:SwapIfNeeded();
	RS3GUI:UpdateTab(2);
end

function RSwapper3:SetBlind()
	if (RSwapper3.db.char.Blind) then
		RSwapper3:Print(L["Announcing via say when you cast Blind is |cffff0000disabled|r."])
		RSwapper3:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	else
		RSwapper3:Print(L["Announcing via say when you cast Blind is |cff00ff00enabled|r."])
		RSwapper3:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	end
	RSwapper3.db.char.Blind = not RSwapper3.db.char.Blind;
	RS3GUI:UpdateTab(2);
end

function RSwapper3:HasBuff(unit, bufflist)
	local i, curbuff = 1, nil;

	curbuff = UnitBuff(unit, i);
	while (curbuff) do
		if (strfind(bufflist, curbuff)) then
			return true;
		end
		i = i + 1;
		curbuff = UnitBuff(unit, i);
	end
	return false;
end

function RSwapper3:HasDebuff(unit, debufflist)
	local i, curdebuff = 1, nil;

	curdebuff = UnitDebuff(unit, i);
	while (curdebuff) do
		if (strfind(debufflist, curdebuff)) then
			return true;
		end
		i = i + 1;
		curdebuff = UnitDebuff(unit, i);
	end
	return false;
end

function RSwapper3:SwapIfNeeded()
	local found, link, broken, ofound, olink, obroken, maintype, ssform, usedag, _;

	-- check if enabled
	if (not RSwapper3.db.char.Enable) then
		dataobj.text = L["|cffff0000[Disabled]|r"];
		RSwapper3.DOTip = L["RSwapper3 has been |cffff0000disabled|r for this character."];
		return;
	end

	-- check if we're channeling
	if (UnitChannelInfo("player")) then
		return;
	end

	-- check if fishing
	link = GetInventoryItemLink("player", RSwapper3.nMAINHANDSLOT);
	if (link) then
		_, _, _, _, _, _, maintype = GetItemInfo(link);
		if (maintype == RSwapper3.LFISHINGPOLES) then
			dataobj.text = RSwapper3.ISFISHING;
			RSwapper3.DOTip = nil;
			RSwapper3.LASTERR = 3;
			RSwapper3.LASTERRT = GetTime();
			return;
		end
	end

	-- check if we have any stuns (preventing us from swapping)
	if (RSwapper3:HasDebuff("player", LPLAYERSTUNS)) then
		return;
	end

	-- First, obviously, check if we're stealthed
	ssform = GetShapeshiftForm(nil);
	usedag = ((ssform == 1) or (ssform == 2)) or
	-- check for Shadow Dance
		(RSwapper3.db.char.Dance and (ssform == 3)) or
	-- Finally, check target debuffs, if we also swap when target is stunned	 
		((not RSwapper3.db.char.Stealth) and RSwapper3:HasDebuff("target", LTARGETSTUNS));

	if (usedag) then
		found, link, broken = RSwapper3:WeaponExists(RSwapper3.db.char.Dagger);
		ofound, olink, obroken = RSwapper3:WeaponExists(RSwapper3.db.char.ODagger, true);
		if ((found == -3) or (ofound == -3)) then
			-- Do nothing.
		elseif (found == -2) then
			dataobj.text = RSwapper3.NEEDDAGGER;
			RSwapper3.DOTip = L["|cffff0000Dagger was not set!|r"];
			RSwapper3:SetError(-12, 5, RSwapper3.DOTip);
		elseif (found == -1) then
			GameTooltip:SetHyperlink(self.db.char.Dagger);
			dataobj.text = RSwapper3.NEEDDAGGER;
			RSwapper3.DOTip = format(L["%s |cffff0000does not appear to be a weapon!|r"], RSwapper3.db.char.Dagger);
			RSwapper3:SetError(-11, 5, RSwapper3.DOTip);
		elseif (ofound == -1) then
			GameTooltip:SetHyperlink(self.db.char.ODagger);
			dataobj.text = RSwapper3.NEEDDAGGER;
			RSwapper3.DOTip = format(SECONDARYHANDSLOT .. ": " .. L["%s |cffff0000does not appear to be a weapon!|r"], RSwapper3.db.char.ODagger);
			RSwapper3:SetError(-11, 5, RSwapper3.DOTip);
		elseif (found == 0) then
			dataobj.text = RSwapper3.NEEDDAGGER;
			RSwapper3.DOTip = format(L["|cffff0000Could not find dagger:|r %s"], RSwapper3.db.char.Dagger);
			RSwapper3:SetError(10, 5, RSwapper3.DOTip);
		elseif (ofound == 0) then
			dataobj.text = RSwapper3.NEEDDAGGER;
			RSwapper3.DOTip = format(SECONDARYHANDSLOT .. ": " .. L["|cffff0000Could not find dagger:|r %s"], RSwapper3.db.char.ODagger);
			RSwapper3:SetError(10, 5, RSwapper3.DOTip);
		elseif (found == 4 and ofound == -2) then
			dataobj.text = RSwapper3.NEEDDAGGER;
			RSwapper3.DOTip = format(L["|cffff0000Can not swap|r %s |cffff0000from the off hand when the current main hand is Main Hand restricted!|r"], link);
			RSwapper3:SetError(14, 5, RSwapper3.DOTip);
		elseif (broken) then
			dataobj.text = RSwapper3.NEEDDAGGER;
			RSwapper3.DOTip = format(L["%s |cffff0000is broken! Not swapping.|r"], link);
			RSwapper3:SetError(15, 5, RSwapper3.DOTip);
		elseif (obroken) then
			dataobj.text = RSwapper3.NEEDDAGGER;
			RSwapper3.DOTip = format(SECONDARYHANDSLOT .. ": " .. L["%s |cffff0000is broken! Not swapping.|r"], olink);
			RSwapper3:SetError(15, 5, RSwapper3.DOTip);
		elseif (((found >= 2) and (found <= 4)) or ((ofound >= 2) and (ofound <= 3))) then
			dataobj.text = RSwapper3.NEEDDAGGER;
			RSwapper3.DOTip = format(L["Swap to %s is pending."], link);
-- if we have an off-hand, but main-hand is main hand restricted
-- then swap the off-hand first, so main-hand goes in the right bag slot
			if ((ofound ~= -2) and (found == 4)) then
				EquipItemByName(olink, RSwapper3.nSECONDARYHANDSLOT);
				EquipItemByName(link, RSwapper3.nMAINHANDSLOT);
-- else, swap the main-hand first, then the off-hand
			elseif (ofound ~= -2) then
				EquipItemByName(link, RSwapper3.nMAINHANDSLOT);
				EquipItemByName(olink, RSwapper3.nSECONDARYHANDSLOT);
-- else, just swap the main-hand
			else
				EquipItemByName(link, RSwapper3.nMAINHANDSLOT);
			end
			RSwapper3.LASTERR = 1;
			RSwapper3.LASTERRT = GetTime();
		elseif (found == 1 and (ofound == 1 or ofound == -2)) then
			dataobj.text = RSwapper3.HAVEDAGGER;
			RSwapper3.DOTip = format(L["%s is equipped."], link);
			if (RSwapper3.LASTERR == 1) then
				RSwapper3:PlaySound(Sounds.Swap)
			end
			RSwapper3.LASTERR = 0;
		end
	else
		found, link, broken = RSwapper3:WeaponExists(RSwapper3.db.char.Sword);
		ofound, olink, obroken = RSwapper3:WeaponExists(RSwapper3.db.char.OSword, true, link);
		if ((found == -3) or (ofound == -3)) then
			-- Do nothing.
		elseif (found == -2) then
			dataobj.text = RSwapper3.NEEDSWORD;
			RSwapper3.DOTip = L["|cffff0000Sword was not set!|r"];
			RSwapper3:SetError(-22, 5, RSwapper3.DOTip);
		elseif (found == -1) then
			GameTooltip:SetHyperlink(self.db.char.Sword);
			dataobj.text = RSwapper3.NEEDSWORD;
			RSwapper3.DOTip = format(L["%s |cffff0000does not appear to be a weapon!|r"], RSwapper3.db.char.Sword);
			RSwapper3:SetError(-21, 5, RSwapper3.DOTip);
		elseif (ofound == -1) then
			GameTooltip:SetHyperlink(self.db.char.OSword);
			dataobj.text = RSwapper3.NEEDSWORD;
			RSwapper3.DOTip = format(SECONDARYHANDSLOT .. ": " .. L["%s |cffff0000does not appear to be a weapon!|r"], RSwapper3.db.char.OSword);
			RSwapper3:SetError(-21, 5, RSwapper3.DOTip);
		elseif (found == 0) then
			dataobj.text = RSwapper3.NEEDSWORD;
			RSwapper3.DOTip = format(L["|cffff0000Could not find sword:|r %s"], RSwapper3.db.char.Sword);
			RSwapper3:SetError(20, 5, RSwapper3.DOTip);
		elseif (ofound == 0) then
			dataobj.text = RSwapper3.NEEDSWORD;
			RSwapper3.DOTip = format(SECONDARYHANDSLOT .. ": " .. L["|cffff0000Could not find sword:|r %s"], RSwapper3.db.char.OSword);
			RSwapper3:SetError(20, 5, RSwapper3.DOTip);
		elseif (found == 4 and ofound == -2) then
			dataobj.text = RSwapper3.NEEDSWORD;
			RSwapper3.DOTip = format(L["|cffff0000Can not swap|r %s |cffff0000from the off hand when the current main hand is Main Hand restricted!|r"], link);
			RSwapper3:SetError(24, 5, RSwapper3.DOTip);
		elseif (broken) then
			dataobj.text = RSwapper3.NEEDSWORD;
			RSwapper3.DOTip = format(L["%s |cffff0000is broken! Not swapping.|r"], link);
			RSwapper3:SetError(25, 5, RSwapper3.DOTip);
		elseif (obroken) then
			dataobj.text = RSwapper3.NEEDSWORD;
			RSwapper3.DOTip = format(SECONDARYHANDSLOT .. ": " .. L["%s |cffff0000is broken! Not swapping.|r"], olink);
			RSwapper3:SetError(25, 5, RSwapper3.DOTip);
		elseif (((found >= 2) and (found <= 4)) or ((ofound >= 2) and (ofound <= 3))) then
			dataobj.text = RSwapper3.NEEDSWORD;
			RSwapper3.DOTip = format(L["Swap to %s is pending."], link);
-- if we have an off-hand, but main-hand is main hand restricted
-- then swap the off-hand first, so main-hand goes in the right bag slot
			if ((ofound ~= -2) and (found == 4)) then
				EquipItemByName(olink, RSwapper3.nSECONDARYHANDSLOT);
				EquipItemByName(link, RSwapper3.nMAINHANDSLOT);
-- else, swap the main-hand first, then the off-hand
			elseif (ofound ~= -2) then
				EquipItemByName(link, RSwapper3.nMAINHANDSLOT);
				EquipItemByName(olink, RSwapper3.nSECONDARYHANDSLOT);
-- else, just swap the main-hand
			else
				EquipItemByName(link, RSwapper3.nMAINHANDSLOT);
			end
			RSwapper3.LASTERR = 2;
			RSwapper3.LASTERRT = GetTime();
		elseif (found == 1 and (ofound == 1 or ofound == -2)) then
			dataobj.text = RSwapper3.HAVESWORD;
			RSwapper3.DOTip = format(L["%s is equipped."], link);
			if (RSwapper3.LASTERR == 2) then
				RSwapper3:PlaySound(Sounds.Swap)
			end
			RSwapper3.LASTERR = 0;
		end
	end
end

function RSwapper3:UNIT_AURA(event, unit)
	if ((unit == "player") or (unit == "target") or (unit == "playertarget")) then
		RSwapper3:SwapIfNeeded();
	end
end

function RSwapper3:UNIT_INVENTORY_CHANGED(info, unit)
	if (unit == "player") then
		RSwapper3:SwapIfNeeded(true);
	end
end

function RSwapper3:UPDATE_SHAPESHIFT_FORM()
	RSwapper3:SwapIfNeeded();
end

function RSwapper3:PLAYER_TARGET_CHANGED(event, change)
	RSwapper3:SwapIfNeeded();
end

function RSwapper3:COMBAT_LOG_EVENT_UNFILTERED(info, timestamp, event, sourceguid, ...)
	if (sourceguid == PlayerGUID) then
		if (event == "SPELL_AURA_APPLIED" and ((select(7, ...)) == RSwapper3.LBLIND)) then
			SendChatMessage(gsub(L["Casting Blind on %t. Don't attack and break it."], "%%t", select(4, ...)), "SAY");
		end
	end
end

function RSwapper3:SetError(err, elap, str)
	if ((RSwapper3.LASTERR ~= err) or ((GetTime() - RSwapper3.LASTERRT) > elap)) then
		RSwapper3.LASTERR = err;
		RSwapper3.LASTERRT = GetTime();
		if (str) then
			RSwapper3:Print(str);
			RSwapper3:PlaySound(Sounds.Error);
		end
		return true;
	end
	return false;
end

function RSwapper3:PlaySound(sound)
	if (sound == nil) then
		return;
	end
	if (RSwapper3.db.char.Sound) then
		PlaySoundFile(sound);
	end
end
